/*
 *	MEM.LDG : Gestionnaire de mmoire pour LDG
 *	Dominique Brziat (c) 1998-2000
 *
 *	Gestionnaire de mmoire  l'attention des
 *	libraries dynamiques. Cette librairie implmente
 *	les fonctions malloc() realloc(), etc ... la
 *	mmoire est partagable.
 *	Ce code vient de la MiNTLib
 *	longueur de la tabulation : 4
 *
 *	Version Pure C.
 *
 *  Historique : 
 *		v 0.10 : premiere version
 *		v 0.20 : la librairie est maintenant LDG_NOT_SHARED
 *				 pour rsoudre les problmes dues aux partages
 *				 de zone mmoire par plusieurs applications dans
 *				 le cas ou le premier client se termine. 
 *				 le module LDGMAIN est linke en premier, sinon
 *				 sous MiNT, la premiere string est crase, je
 *				 comprend d'ailleurs pas pourquoi.
 *		v 0.30 : appel a Mxalloc() avec le bit 14 actif
 *				 librarie de nouveau partageable. : c'est quoi deja ca ?
 *		v 0.31 : ajout de ldg_strdup()
 *		v 0.40 : on supprime le bit 14. Rappel : ce bit permet que
 *               le bloc ne soit pas supprimable de la mmoire (sauf
 *				 pour l'application qui a fait le malloc). Or, en cas de
 *				 plantage de l'appli. On ne peut plus le librer (ref
 *				 doc MiNT). 
 *				 une solution : bloquer les blocs par le dmon LDG (qui
 *				 intgre ca depuis LDG version 2.30.
 *               - la fonction realloc tait mal prototyp dans mem.ldg
 *
 *		v 0.50 : mem.ldg alloue maintenant un bloc personnel par client.
 *				 cela permet :
 *					- de resoudre les problemes precedents car chaque
 *					  possde son propre bloc.
 *					- d'utiliser de la mmoire non partageable
 *					- si un client merde en dpassant la mmoire qui
 *					  lui est allou, il n'empietera pas sur celle d'un
 *					  autre client.
 *				 Si un client souhaite partager de la mmoire, il doit le
 *				 faire explicitement (avec les mcanismes habituels de
 *				 MiNT). => un exemple dans la doc. ou faire des fonctions
 *				 haut niveau ?
 *				 Le prix de cette meilleur gestion de la mmoire est que les
 *				 fonctions possede un parametre supplementaire (toutefois il
 *				 masqu dans le binding des fonctions).
 *
 */

#define VERSION		"0.50"
#define NUM_VER		0x0050

/* Fichiers headers */

/*#include <portab.h>*/
#include <gem.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ldg.h>
#include "globals.h"

/* Macros */
#define max(a,b)	(((a)>(b))?(a):(b))

/* Prototypages */
void* __CDECL ldg_malloc( unsigned long size, long context);
void* __CDECL ldg_calloc( unsigned long, unsigned long, long context);
void* __CDECL ldg_realloc( void *adr, unsigned long size, long context);
void  __CDECL ldg_free( void *adr, long context);
void  __CDECL ldg_malloczero( long yes, long context);
void  __CDECL ldg_mallocChunkSize( unsigned long size, long context);
char* __CDECL ldg_strdup( char *, long context);

/* long CDECL  getusedmem( long context); */

void __CDECL ldg_allocmode( long mode, long context) {
	extern short mxalloc[];
	mxalloc[context] = mode ? 1:0;
}

/* Variables globales */
static PROC Proc[]  = {
"ldg_calloc", "/* Memory reservation with initialization */\n"
			  "void* ldg_calloc( unsigned long num, unsigned long size, long context);\n", (void *)ldg_calloc,

"ldg_free", "/* Memory release */\n"
			  "void ldg_free( void *addr, long context);\n", (void *)ldg_free,

"ldg_malloczero", 
			  "/* ldg_malloc() set memory to zero */\n"
			  "void ldg_malloczero( long yes, long context);\n", (void *)ldg_malloczero,

"ldg_mallocChunkSize", 
			  "/* Ajust memory parameters */\n"
			  "void ldg_mallocChunkSize( unsigned long size, long context);\n", (void *)ldg_mallocChunkSize,

"ldg_malloc", "/* Memory reservation */\n"
			  "void* ldg_malloc( unsigned long size, long context);\n", (void *)ldg_malloc,

"ldg_realloc", "/* Memory reallocation */\n"
			  "void* ldg_realloc( void* adr, unsigned long size, long context);\n", (void *)ldg_realloc,

"ldg_strdup", "/* String duplication */\n"
			  "char* ldg_strdup( char *srd, long context);\n", (void *)ldg_strdup, 

"ldg_allocmode", "/* Choice memory allocation mode */\n"
				 "void ldg_allocmode( long mode, long context);", (void*)ldg_allocmode 

			  };

static char Info[] = "Memory manager for LDG by Dominique Brziat version " VERSION " ( " __DATE__ " at " __TIME__ ").";

LDGLIB Ldg[] = { NUM_VER, 8, Proc, Info, 0};

extern int ldg_context; 

void* __CDECL ldg_malloc( unsigned long mem, long context) {
	void *my_malloc( unsigned long);
	void *adr;
	ldg_context = (int)context;
	adr = my_malloc( mem);
	return adr;
}

void* __CDECL ldg_calloc( unsigned long num, unsigned long size, long context) {
	void *my_calloc( unsigned long, unsigned long);
	ldg_context = (int)context;
	return my_calloc( num, size);
}

void* __CDECL ldg_realloc( void* adr, unsigned long mem, long context) {
	void *my_realloc( void *, unsigned long);
	ldg_context = (int)context;
	return my_realloc( adr, mem);
}

void __CDECL ldg_free( void* addr, long context) {
	void my_free( void *);
	ldg_context = (int)context;
	my_free( addr);
}

void __CDECL ldg_malloczero( long yes, long context) {
	void _malloczero( int);
	ldg_context = (int)context;
	_malloczero( (int)yes);
}

void __CDECL ldg_mallocChunkSize( unsigned long size, long context) {
	void _mallocChunkSize ( size_t siz);
	ldg_context = (int)context;
	_mallocChunkSize( (size_t)size);
}

char* __CDECL ldg_strdup( char *str, long context) {
	void *my_malloc( unsigned long);
	ldg_context = (int)context;
	if( str && *str) {
		char *p = my_malloc( (strlen( str)+1)*sizeof(char));
		strcpy( p, str);
		return p;
	} else
		return NULL;
}

void init_global_vars(void);

int main( void) {
	extern int __mint, __magic;

	ldg_cookie( 'MiNT', (long*)&__mint);
	__magic = ldg_cookie( 'MagX', NULL);
	init_global_vars();
	if( ldg_init( Ldg)) 
		puts( "Warning : this program is a LDG-library and cannot be launched from desktop");
	return 0;
}

